MODULE sbcmod_tam
#ifdef key_tam
   !!======================================================================
   !!                       ***  MODULE  sbcmod_tam  ***
   !! Surface module :  provide to the ocean its surface boundary condition
   !!                       Tangent&Adjoint module
   !!======================================================================
   !! History of the direct module :  
   !!            3.0   !  2006-07  (G. Madec)  Original code
   !! History of the T&A module :
   !!            3.0   !  2008-11  (A. Vidard) TAM of the 2006-07 version
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   sbc_init       : read namsbc namelist
   !!   sbc            : surface ocean momentum, heat and freshwater boundary conditions
   !!----------------------------------------------------------------------
   USE par_kind      , ONLY: & ! Precision variables
      & wp
   USE phycst        , ONLY: & !
      & rday
   !   USE ice_oce         ! sea-ice model : LIM
   USE dom_oce       , ONLY: & ! ocean space domain variables
      & rdt,                 &
      & cp_cfg
   USE sbc_oce       , ONLY: & ! Surface boundary condition: ocean fields
      & nn_fsbc
   USE sbc_oce_tam   , ONLY: & ! Surface boundary condition: ocean fields
      & fr_i_tl
   USE sbcssm_tam    , ONLY: & ! surface boundary condition: sea-surface mean variables
      & sbc_ssm_tan,         &
      & sbc_ssm_adj,         &
      & sbc_ssm_adj_tst
   USE sbcana_tam, ONLY:  & ! surface boundary condition: analytical formulation
      & sbc_gyre_tan,       &
      & sbc_gyre_adj,       &
      & sbc_gyre_adj_tst
!*B      & sbc_ana_tan,       &
!*B      & sbc_ana_adj,       &
!*B      & sbc_ana_adj_tst
   USE sbcflx_tam    , ONLY: & ! surface boundary condition: flux formulation
      & sbc_flx_tan,         &
      & sbc_flx_adj,         &
      & sbc_flx_adj_tst
!   USE sbcblk_clio     ! surface boundary condition: bulk formulation : CLIO
!   USE sbcblk_core     ! surface boundary condition: bulk formulation : CORE
!   USE sbcice_if       ! surface boundary condition: ice-if sea-ice model
!   USE sbcice_lim      ! surface boundary condition: LIM 3.0 sea-ice model
!   USE sbcice_lim_2    ! surface boundary condition: LIM 2.0 sea-ice model
!   USE sbccpl          ! surface boundary condition: coupled florulation
   USE sbcssr_tam    , ONLY: & ! surface boundary condition: sea surface restoring
      & sbc_ssr_tan,         &
      & sbc_ssr_adj,         &
      & sbc_ssr_adj_tst
   USE closea_tam    , ONLY: & !specific treatments associated with closed seas
      & sbc_clo_tan,         &
      & sbc_clo_adj,         &
      & sbc_clo_adj_tst
!   USE sbcrnf_tam          ! surface boundary condition: runoffs
   USE sbcrnf        , ONLY: & ! surface boundary condition: runoffs
      & rnfmsk,              &
      & rnfmsk_z,            &
      & ln_rnf_mouth,        &
      & nkrnf
   USE sbcfwb_tam    , ONLY: & ! surface boundary condition: freshwater budget
      & sbc_fwb_tan,         &
      & sbc_fwb_adj,         &
      & sbc_fwb_adj_tst
   USE closea        , ONLY: & ! closed sea
      & nclosea
   USE in_out_manager, ONLY: & ! I/O manager 
      & lwp,                 &
      & ctl_warn,            &
      & numout,              &
      & numnam,              & 
      & nit000,              & 
      & nitend,              &
      & nstop

   IMPLICIT NONE
   PRIVATE

   PUBLIC   sbc_tan    ! routine called by step_tam.F90
   PUBLIC   sbc_adj    ! routine called by step_tam.F90
   PUBLIC   sbc_adj_tst! routine called by tst.F90
   
   !! * namsbc namelist (public variables)
   LOGICAL , PUBLIC ::   ln_ana      = .FALSE.   !: analytical boundary condition flag
   LOGICAL , PUBLIC ::   ln_flx      = .FALSE.   !: flux      formulation
   LOGICAL , PUBLIC ::   ln_blk_clio = .FALSE.   !: CLIO bulk formulation
   LOGICAL , PUBLIC ::   ln_blk_core = .FALSE.   !: CORE bulk formulation
   LOGICAL , PUBLIC ::   ln_cpl      = .FALSE.   !: coupled   formulation (overwritten by key_sbc_coupled )
   LOGICAL , PUBLIC ::   ln_dm2dc    = .FALSE.   !: Daily mean to Diurnal Cycle short wave (qsr)
   LOGICAL , PUBLIC ::   ln_rnf      = .FALSE.   !: runoffs / runoff mouths
   LOGICAL , PUBLIC ::   ln_ssr      = .FALSE.   !: Sea Surface restoring on SST and/or SSS      
   INTEGER , PUBLIC ::   nn_ice      = 0         !: flag on ice in the surface boundary condition (=0/1/2)
   INTEGER , PUBLIC ::   nn_fwb      = 0         !: type of FreshWater Budget control (=0/1/2)
   INTEGER          ::   nn_ico_cpl  = 0         !: ice-ocean coupling indicator
   !                                             !  = 0   LIM-3 old case
   !                                             !  = 1   stresses computed using now ocean velocity
   !                                             !  = 2   combination of 0 and 1 cases

   INTEGER ::   nsbc   ! type of surface boundary condition (deduced from namsbc informations)
   INTEGER ::   nice   ! type of ice in the surface boundary condition (deduced from namsbc informations)
   LOGICAL :: lfirst = .TRUE. ! initialisation flag
      
   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.0 , LOCEAN-IPSL (2008) 
   !! $Id: sbcmod.F90 1172 2008-09-10 15:32:47Z ctlod $
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE sbc_init_tam
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE sbc_init_tam ***
      !!
      !! ** Purpose of the direct routine:
      !!           Initialisation of the ocean surface boundary computation
      !!
      !! ** Method  :   Read the namsbc namelist and set derived parameters
      !!
      !! ** Action  : - read namsbc parameters
      !!              - nsbc: type of sbc
      !!----------------------------------------------------------------------
      INTEGER ::   icpt      ! temporary integer
      !!
      NAMELIST/namsbc/ nn_fsbc, ln_ana, ln_flx, ln_blk_clio, ln_blk_core, ln_cpl,   &
         &             nn_ice , ln_dm2dc, ln_rnf, ln_ssr, nn_fwb, nn_ico_cpl
      !!----------------------------------------------------------------------

      IF (lfirst) THEN
         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) 'sbc_init_tam : surface boundary condition setting'
            WRITE(numout,*) '~~~~~~~~~~~~ '
         ENDIF

         REWIND ( numnam )                   ! Read Namelist namsbc
         READ   ( numnam, namsbc )
      ! overwrite namelist parameter using CPP key information
!!gmhere no overwrite, test all option via namelist change: require more incore memory
!!gm  IF( lk_sbc_cpl       ) THEN   ;   ln_cpl      = .TRUE.   ;   ELSE   ;   ln_cpl      = .FALSE.   ;   ENDIF
!*B      IF( lk_lim2 )            nn_ice      = 2
!*B      IF( lk_lim3 )            nn_ice      = 3
!*B      IF( cp_cfg == 'gyre' ) THEN
!*B          ln_ana      = .TRUE.   
!*B          nn_ice      =   0
!*B      ENDIF
      
      ! Control print
         IF(lwp) THEN
            WRITE(numout,*) '        Namelist namsbc (overwritten using CPP key defined)'
            WRITE(numout,*) '           frequency update of sbc (and ice)             nn_fsbc     = ', nn_fsbc
            WRITE(numout,*) '           Type of sbc : '
            WRITE(numout,*) '              analytical formulation                     ln_ana      = ', ln_ana
            WRITE(numout,*) '              flux       formulation                     ln_flx      = ', ln_flx
!*B         WRITE(numout,*) '              CLIO bulk  formulation                     ln_blk_clio = ', ln_blk_clio
!*B         WRITE(numout,*) '              CORE bulk  formulation                     ln_blk_core = ', ln_blk_core
!*B         WRITE(numout,*) '              coupled    formulation (T if key_sbc_cpl)  ln_cpl      = ', ln_cpl
            WRITE(numout,*) '           Misc. options of sbc : '
!*B         WRITE(numout,*) '              ice management in the sbc (=0/1/2/3)       nn_ice      = ', nn_ice 
!*B         WRITE(numout,*) '              ice-ocean stress computation (=0/1/2)      nn_ico_cpl  = ', nn_ico_cpl
            WRITE(numout,*) '              daily mean to diurnal cycle qsr            ln_dm2dc    = ', ln_dm2dc 
            WRITE(numout,*) '              runoff / runoff mouths                     ln_rnf      = ', ln_rnf
            WRITE(numout,*) '              Sea Surface Restoring on SST and/or SSS    ln_ssr      = ', ln_ssr
            WRITE(numout,*) '              FreshWater Budget control  (=0/1/2)        nn_fwb      = ', nn_fwb
            WRITE(numout,*) '              closed sea (=0/1) (set in namdom)          nclosea     = ', nclosea
         ENDIF

         IF( .NOT. ln_rnf )   THEN                      ! no specific treatment in rivers mouths vicinity
            ln_rnf_mouth = .FALSE.                      
            nkrnf = 0
            rnfmsk(:,:) = 0.e0
            rnfmsk_z(:) = 0.e0
         ENDIF
         IF( nn_ice == 0  )   fr_i_tl(:,:) = 0.e0          ! no ice in the domain, ice fraction is always zero

      ! Check consistancy  

!*B      IF( nn_ice == 2 )   THEN
!*B         IF( MOD( nitend - nit000 + 1, nn_fsbc) /= 0 ) THEN 
!*B            WRITE(ctmp1,*) 'experiment length (', nitend - nit000 + 1, ') is NOT a multiple of nn_fsbc (', nn_fsbc, ')'
!*B            CALL ctl_stop( ctmp1, 'Impossible to do proper restart files' )
!*B         ENDIF
!*B         IF( MOD( nstock, nn_fsbc) /= 0 ) THEN 
!*B            WRITE(ctmp1,*) 'nstock ('           , nstock             , ') is NOT a multiple of nn_fsbc (', nn_fsbc, ')'
!*B            CALL ctl_stop( ctmp1, 'Impossible to do proper restart files' )
!*B         ENDIF
!*B      ENDIF

         IF( MOD( rday, nn_fsbc*rdt ) /= 0 )   CALL ctl_warn( 'nn_fsbc is NOT a multiple of the number of time steps in a day' )

!*B      IF( nn_ice == 2 .AND. .NOT.( ln_blk_clio .OR. ln_blk_core ) )   &
!*B         &   CALL ctl_stop( 'sbc_init: sea-ice model requires a bulk formulation' )
      
      ! Choice of the Surface Boudary Condition (set nsbc)
         icpt = 0
         IF( ln_ana          ) THEN   ;   nsbc =  1   ; icpt = icpt + 1   ;   ENDIF       ! analytical      formulation
         IF( ln_flx          ) THEN   ;   nsbc =  2   ; icpt = icpt + 1   ;   ENDIF       ! flux            formulation
         IF( ln_blk_clio     ) THEN   ;   nsbc =  3   ; icpt = icpt + 1   ;   ENDIF       ! CLIO bulk       formulation
         IF( ln_blk_core     ) THEN   ;   nsbc =  4   ; icpt = icpt + 1   ;   ENDIF       ! CORE bulk       formulation
         IF( ln_cpl          ) THEN   ;   nsbc =  5   ; icpt = icpt + 1   ;   ENDIF       ! Coupled         formulation
         IF( cp_cfg == 'gyre') THEN   ;   nsbc =  0                       ;   ENDIF       ! GYRE analytical formulation

         IF( icpt /= 1  ) THEN
            WRITE(numout,*)
            WRITE(numout,*) '           E R R O R in setting the sbc, one and only one namelist/CPP key option '
            WRITE(numout,*) '                     must be choosen. You choose ', icpt, ' option(s)'
            WRITE(numout,*) '                     We stop'
            nstop = nstop + 1
         ENDIF

         IF( .NOT.((nsbc /= 2) .OR. (nsbc /= 0)) ) THEN
            WRITE(numout,*)
            WRITE(numout,*) '           E R R O R in setting the sbc, only ln_flx and cp_gyre available '
            STOP
         ENDIF
         IF(lwp) THEN
            WRITE(numout,*)
            IF( nsbc ==  0 )   WRITE(numout,*) '              GYRE analytical formulation'
            IF( nsbc ==  1 )   WRITE(numout,*) '              analytical formulation'
            IF( nsbc ==  2 )   WRITE(numout,*) '              flux formulation'
            IF( nsbc ==  3 )   WRITE(numout,*) '              CLIO bulk formulation'
            IF( nsbc ==  4 )   WRITE(numout,*) '              CORE bulk formulation'
            IF( nsbc ==  5 )   WRITE(numout,*) '              coupled formulation'
         ENDIF
         lfirst = .FALSE.
      END IF
      !
   END SUBROUTINE sbc_init_tam


   SUBROUTINE sbc_tan( kt )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE sbc_tan  ***
      !!              
      !! ** Purpose of the direct routine:
      !!                provide at each time-step the ocean surface boundary
      !!                condition (momentum, heat and freshwater fluxes)
      !!
      !! ** Method  :   blah blah  to be written ????????? 
      !!                CAUTION : never mask the surface stress field (tke sbc)
      !!
      !! ** Action  : - set the ocean surface boundary condition, i.e.  
      !!                utau, vtau, qns, qsr, emp, emps, qrp, erp
      !!              - updte the ice fraction : fr_i
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt       ! ocean time step
      !!---------------------------------------------------------------------

      IF( kt == nit000 )   CALL sbc_init_tam         ! Read namsbc namelist : surface module

      ! ocean to sbc mean sea surface variables (ss._m)
      ! ---------------------------------------
      CALL sbc_ssm_tan( kt )                         ! sea surface mean currents (at U- and V-points), 
      !                                          ! temperature and salinity (at T-point) over nf_sbc time-step
      !                                          ! (i.e. sst_m, sss_m, ssu_m, ssv_m)

      ! sbc formulation
      ! ---------------
         
      SELECT CASE( nsbc )                        ! Compute ocean surface boundary condition
      !                                          ! (i.e. utau,vtau, qns, qsr, emp, emps)
      CASE(  0 )   ;   CALL sbc_gyre_tan    ( kt )      ! analytical formulation : GYRE configuration
         ! no! in default
      !CASE(  1 )   ;   CALL sbc_ana_tan     ( kt )      ! analytical formulation : uniform sbc
      CASE(  2 )   ;   CALL sbc_flx_tan     ( kt )      ! flux formulation
      !CASE(  3 )   ;   CALL sbc_blk_clio_tan( kt )      ! bulk formulation : CLIO for the ocean
      !CASE(  4 )   ;   CALL sbc_blk_core_tan( kt )      ! bulk formulation : CORE for the ocean
      !CASE(  5 )   ;   CALL sbc_cpl_tan     ( kt )      ! coupled formulation
      END SELECT

      ! Misc. Options
      ! -------------
      ! not available
!*B      SELECT CASE( nn_ice )                                     ! Update heat and freshwater fluxes over ice-covered areas
!*B      CASE(  1 )   ;       CALL sbc_ice_if_tan ( kt )                     ! Ice-cover climatology ("Ice-if" model)
!*B         !                                                      
!*B      CASE(  2 )   ;       CALL sbc_ice_lim_2_tan( kt, nsbc )             ! LIM 2.0 ice model
!*B         !                                                     
!*B      CASE(  3 )   ;       CALL sbc_ice_lim_tan  ( kt, nsbc, nn_ico_cpl)  ! LIM 3.0 ice model
!*B      END SELECT                                              

      ! add runoffs to fresh water fluxes... not needed in tangent
 
      IF( ln_ssr       )   CALL sbc_ssr_tan( kt )                   ! add SST/SSS damping term

      IF( nn_fwb  /= 0 )   CALL sbc_fwb_tan( kt, nn_fwb, nn_fsbc )  ! control the freshwater budget

      IF( nclosea == 1 )   CALL sbc_clo_tan( kt )                   ! treatment of closed sea in the model domain 
      !                                                         ! (update freshwater fluxes)
      !
      !
   END SUBROUTINE sbc_tan

   !!======================================================================
   SUBROUTINE sbc_adj( kt )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE sbc_adj  ***
      !!              
      !! ** Purpose of the direct routine:
      !!                provide at each time-step the ocean surface boundary
      !!                condition (momentum, heat and freshwater fluxes)
      !!
      !! ** Method  :   blah blah  to be written ????????? 
      !!                CAUTION : never mask the surface stress field (tke sbc)
      !!
      !! ** Action  : - set the ocean surface boundary condition, i.e.  
      !!                utau, vtau, qns, qsr, emp, emps, qrp, erp
      !!              - updte the ice fraction : fr_i
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt       ! ocean time step
      !!---------------------------------------------------------------------

      IF( kt == nitend )   CALL sbc_init_tam         ! Read namsbc namelist : surface module

      ! Misc. Options
      ! -------------
      IF( nclosea == 1 )   CALL sbc_clo_adj( kt )                   ! treatment of closed sea in the model domain 
      IF( nn_fwb  /= 0 )   CALL sbc_fwb_adj( kt, nn_fwb, nn_fsbc )  ! control the freshwater budget
      IF( ln_ssr       )   CALL sbc_ssr_adj( kt )                   ! add SST/SSS damping term
      SELECT CASE( nn_ice )                                     ! Update heat and freshwater fluxes over ice-covered areas
!      CASE(  1 )   ;       CALL sbc_ice_if_adj ( kt )                     ! Ice-cover climatology ("Ice-if" model)
         !                                                      
!      CASE(  2 )   ;       CALL sbc_ice_lim_2_adj( kt, nsbc )             ! LIM 2.0 ice model
         !                                                     
!      CASE(  3 )   ;       CALL sbc_ice_lim_adj  ( kt, nsbc, nn_ico_cpl)  ! LIM 3.0 ice model
      END SELECT                                              
      ! sbc formulation
      ! ---------------
         
      SELECT CASE( nsbc )                        ! Compute ocean surface boundary condition
      !                                          ! (i.e. utau,vtau, qns, qsr, emp, emps)
      CASE(  0 )   ;   CALL sbc_gyre_adj    ( kt )      ! analytical formulation : GYRE configuration
!      CASE(  1 )   ;   CALL sbc_ana_adj     ( kt )      ! analytical formulation : uniform sbc
      CASE(  2 )   ;   CALL sbc_flx_adj     ( kt )      ! flux formulation
!      CASE(  3 )   ;   CALL sbc_blk_clio_adj( kt )      ! bulk formulation : CLIO for the ocean
!      CASE(  4 )   ;   CALL sbc_blk_core_adj( kt )      ! bulk formulation : CORE for the ocean
!      CASE(  5 )   ;   CALL sbc_cpl_adj     ( kt )      ! coupled formulation
      END SELECT
      ! ocean to sbc mean sea surface variables (ss._m)
      ! ---------------------------------------
      CALL sbc_ssm_adj( kt )                         ! sea surface mean currents (at U- and V-points), 
      !                                          ! temperature and salinity (at T-point) over nf_sbc time-step
      !                                          ! (i.e. sst_m, sss_m, ssu_m, ssv_m)


   END SUBROUTINE sbc_adj
   SUBROUTINE sbc_adj_tst( kumadt )
      !!-----------------------------------------------------------------------
      !!
      !!                  ***  ROUTINE sbc_adj_tst ***
      !!
      !! ** Purpose : Test the adjoint routine.
      !!
      !! ** Method  : Verify the scalar product 
      !!           
      !!                 ( L dx )^T W dy  =  dx^T L^T W dy
      !!
      !!              where  L   = tangent routine
      !!                     L^T = adjoint routine
      !!                     W   = diagonal matrix of scale factors
      !!                     dx  = input perturbation (random field)
      !!                     dy  = L dx 
      !!
      !!-----------------------------------------------------------------------
      !! * Modules used

      !! * Arguments
      INTEGER, INTENT(IN) :: &
         & kumadt             ! Output unit
      CALL sbc_init_tam         ! Read namsbc namelist : surface module      
!
      CALL sbc_fwb_adj_tst( kumadt )       ! control the freshwater budget
      CALL sbc_ssr_adj_tst( kumadt )       ! add SST/SSS damping term
!      CALL sbc_rnf_adj_tst( kumadt )       ! add runoffs to fresh water fluxes
!      CALL sbc_ice_if_adj_tst( kumadt )    ! Ice-cover climatology ("Ice-if" model)
!      CALL sbc_ice_lim_2_adj_tst( kumadt ) ! LIM 2.0 ice model
!      CALL sbc_ice_lim_adj_tst( kumadt )   ! LIM 3.0 ice model
      CALL sbc_gyre_adj_tst( kumadt )      ! analytical formulation : GYRE configuration
!      CALL sbc_ana_adj_tst( kumadt )       ! analytical formulation : uniform sbc
      CALL sbc_flx_adj_tst( kumadt )        ! flux formulation
!      CALL sbc_blk_clio_adj_tst( kumadt )  ! bulk formulation : CLIO for the ocean
!      CALL sbc_blk_core_adj_tst( kumadt )  ! bulk formulation : CORE for the ocean
!      CALL sbc_cpl_adj_tst( kumadt )       ! coupled formulation
      CALL sbc_ssm_adj_tst( kumadt )       ! sea surface mean currents (at U- and V-points), 
      IF( nclosea == 1 ) CALL sbc_clo_adj_tst( kumadt )       ! closed seas,

   END SUBROUTINE sbc_adj_tst
   !!======================================================================
#endif
END MODULE sbcmod_tam
