MODULE tradmp_tam
#ifdef key_tam
   !!======================================================================
   !!                       ***  MODULE  tradmp_tam  ***
   !! Ocean physics: internal restoring trend on active tracers (T and S)
   !!                Tangent and Adjoint Module
   !!======================================================================
   !! History of the direct module:
   !!            5.0  !  91-03  (O. Marti, G. Madec)  Original code
   !!                 !  92-06  (M. Imbard)  doctor norme
   !!                 !  96-01  (G. Madec)  statement function for e3
   !!                 !  97-05  (G. Madec)  macro-tasked on jk-slab
   !!                 !  98-07  (M. Imbard, G. Madec) ORCA version
   !!            7.0  !  01-02  (M. Imbard)  cofdis, Original code
   !!            8.1  !  01-02  (G. Madec, E. Durand)  cleaning
   !!            8.5  !  02-08  (G. Madec, E. Durand)  free form + modules
   !!                 !  03-08  (M. Balmaseda)  mods to allow eq. damping
   !! History of the TAM:
   !!                 !  08-09  (A. Vidard) tangent and adjoint module 
   !!                                       of the 03-08 version
   !!----------------------------------------------------------------------
#if   defined key_tradmp   ||   defined key_esopa
   !!----------------------------------------------------------------------
   !!   key_tradmp                                         internal damping
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   tra_dmp      : update the tracer trend with the internal damping
   !!   tra_dmp_init : initialization, namlist read, parameters control
   !!   dtacof_zoom  : restoring coefficient for zoom domain
   !!   dtacof       : restoring coefficient for global domain
   !!   cofdis       : compute the distance to the coastline
   !!----------------------------------------------------------------------
   USE par_kind      , ONLY: & ! Precision variables
      & wp
   USE par_oce       , ONLY: & ! Ocean space and time domain variables
      & jpi,                 &
      & jpj,                 & 
      & jpk,                 &
      & jpim1,               &
      & jpjm1,               &
      & jpkm1,               &
      & jpiglo,              &
      & jpjglo,              &
      & jpizoom,             &
      & jpjzoom
   USE oce_tam       , ONLY: & ! ocean dynamics and tracers variables
      & tb_tl,               &
      & sb_tl,               &
      & ta_tl,               &
      & sa_tl,               &
      & tb_ad,               &
      & sb_ad,               &
      & ta_ad,               &
      & sa_ad      
   USE dom_oce       , ONLY: & ! Ocean space and time domain
      & lzoom,               & 
      & lzoom_e,             &  
      & lzoom_w,             &  
      & lzoom_s,             &  
      & lzoom_n,             &  
      & lzoom_arct,          & 
      & lzoom_anta,          &
      & nperio,              &
      & rdt,                 &
      & mi0,                 &
      & mj0,                 &
      & mi1,                 &
      & mj1,                 &
      & mig,                 &
      & mjg,                 &
      & nldi,                &
      & nldj,                &
      & nlei,                &
      & nlej,                &
      & e1t,                 &
      & e2t,                 &
#if defined key_zco
      & e3t_0,               &
      & gdept_0,             &
#else
      & e3t,                 &
      & gdept,               &
#endif
      & gdept_0,             &
      & umask,               &
      & vmask,               &
      & tmask,               &
      & fmask,               &
      & gphiu,               &
      & glamu,               &
      & gphiv,               &
      & glamv,               &
      & gphit,               &
      & glamt,               &
      & gphif,               &
      & glamf
   USE zdf_oce, ONLY: &! ocean vertical physics
      & avt
   USE in_out_manager, ONLY: & ! I/O manager 
      & ctl_stop,            &
      & lwp,                 &
      & numout,              & 
      & numnam,              & 
      & ctmp1,               & 
      & nit000,              &
      & nitend
   USE phycst        , ONLY: & ! Define parameters for the routines
      & rday,                & 
      & rpi,                 &
      & ra,                  &
      & rad
   USE lib_mpp       , ONLY: & ! distributed memory computing
      & lk_mpp
   USE tradmp        , ONLY : &
      & dtacof_zoom, dtacof,  &
      & cofdis
   USE prtctl        , ONLY: & ! Print control
      & prt_ctl
   USE gridrandom    , ONLY: & ! Random Gaussian noise on grids
      & grid_random
   USE dotprodfld    , ONLY: & ! Computes dot product for 3D and 2D fields
      & dot_product
   USE dtatem        , ONLY: & ! temperature data
      & lk_dtatem
   USE dtasal        , ONLY: & ! salinity data
      & lk_dtasal
   USE zdfmxl        , ONLY: & ! mixed layer depth
      & hmlp
   USE tstool_tam    , ONLY: &
      & prntst_adj,          & !
      & stdt,                & ! stdev for temperature
      & stds                   !           salinity
   USE paresp        , ONLY: &
      & wesp_t,              &
      & wesp_s
   IMPLICIT NONE
   PRIVATE

   PUBLIC tra_dmp_tan      ! routine called by step_tam.F90
   PUBLIC tra_dmp_adj      ! routine called by step_tam.F90
   PUBLIC tra_dmp_adj_tst  ! routine called by tst.F90
#if ! defined key_agrif
   LOGICAL, PUBLIC, PARAMETER ::   lk_tradmp = .TRUE.     !: internal damping flag
#else
   LOGICAL, PUBLIC            ::   lk_tradmp = .TRUE.     !: internal damping flag
#endif
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk) :: &
     & resto          !: restoring coeff. on T and S (s-1)
   
   
   LOGICAL :: lfirst = .TRUE.     !: flag for initialisation
   !!* newtonian damping namelist (mandmp) 
   INTEGER  ::   ndmp   =   -1    ! = 0/-1/'latitude' for damping over T and S
   INTEGER  ::   ndmpf  =    2    ! = 1 create a damping.coeff NetCDF file 
   INTEGER  ::   nmldmp =    0    ! = 0/1/2 flag for damping in the mixed layer
   REAL(wp) ::   sdmp   =   50.   ! surface time scale for internal damping (days)
   REAL(wp) ::   bdmp   =  360.   ! bottom time scale for internal damping (days)
   REAL(wp) ::   hdmp   =  800.   ! depth of transition between sdmp and bdmp (meters)
   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"


CONTAINS

   SUBROUTINE tra_dmp_tan( kt )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE tra_dmp_tan  ***
      !!                  
      !! ** Purpose of direct routine:
      !!      Compute the tracer trend due to a newtonian damping
      !!      of the tracer field towards given data field and add it to the
      !!      general tracer trends.
      !!
      !! ** Method  :   Newtonian damping towards t_dta and s_dta computed 
      !!      and add to the general tracer trends:
      !!                     ta = ta + resto * (t_dta - tb)
      !!                     sa = sa + resto * (s_dta - sb)
      !!         The trend is computed either throughout the water column
      !!      (nlmdmp=0) or in area of weak vertical mixing (nlmdmp=1) or
      !!      below the well mixed layer (nlmdmp=2)
      !!
      !! ** Action  : - update the tracer trends (ta,sa) with the newtonian 
      !!                damping trends.
      !!
      !! ASSUME key_zdfcst_tam
      !!----------------------------------------------------------------------
      !!
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index
      !!
      INTEGER  ::   ji, jj, jk            ! dummy loop indices
      REAL(wp) ::   ztest, ztatl, zsatl       ! temporary scalars
      !!----------------------------------------------------------------------
      IF( kt == nit000 )   CALL tra_dmp_init_tam  ! Initialization

      ! 1. Newtonian damping trends on tracer fields
      ! --------------------------------------------
      !    compute the newtonian damping trends depending on nmldmp

      SELECT CASE ( nmldmp )
      !
      CASE( 0 )                ! newtonian damping throughout the water column
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ztatl = - resto(ji,jj,jk) *  tb_tl(ji,jj,jk) 
                  zsatl = - resto(ji,jj,jk) *  sb_tl(ji,jj,jk) 
                  ! add the trends to the general tracer trends
                  ta_tl(ji,jj,jk) = ta_tl(ji,jj,jk) + ztatl
                  sa_tl(ji,jj,jk) = sa_tl(ji,jj,jk) + zsatl
               END DO
            END DO
         END DO
         !
      CASE ( 1 )                ! no damping in the turbocline (avt > 5 cm2/s)
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ztest = avt(ji,jj,jk) - 5.e-4
      !! ASSUME key_zdfcst_tam
                  IF( ztest < 0. ) THEN
                     ztatl = - resto(ji,jj,jk) * tb_tl(ji,jj,jk) 
                     zsatl = - resto(ji,jj,jk) * sb_tl(ji,jj,jk)
                  ELSE
                     ztatl = 0.e0
                     zsatl = 0.e0
                  ENDIF
                  ! add the trends to the general tracer trends
                  ta_tl(ji,jj,jk) = ta_tl(ji,jj,jk) + ztatl
                  sa_tl(ji,jj,jk) = sa_tl(ji,jj,jk) + zsatl
               END DO
            END DO
         END DO
         !
      CASE ( 2 )                ! no damping in the mixed layer  
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  IF( fsdept(ji,jj,jk) >= hmlp (ji,jj) ) THEN
                     ztatl = - resto(ji,jj,jk) * tb_tl(ji,jj,jk)
                     zsatl = - resto(ji,jj,jk) * sb_tl(ji,jj,jk)
                  ELSE
                     ztatl = 0.e0
                     zsatl = 0.e0
                  ENDIF
                  ! add the trends to the general tracer trends
                  ta_tl(ji,jj,jk) = ta_tl(ji,jj,jk) + ztatl
                  sa_tl(ji,jj,jk) = sa_tl(ji,jj,jk) + zsatl
               END DO
            END DO
         END DO
         !
      END SELECT

   END SUBROUTINE tra_dmp_tan
   SUBROUTINE tra_dmp_adj( kt )
      !!----------------------------------------------------------------------
      !!                   ***  ROUTINE tra_dmp_adj  ***
      !!                  
      !! ** Purpose of direct routine:
      !!      Compute the tracer trend due to a newtonian damping
      !!      of the tracer field towards given data field and add it to the
      !!      general tracer trends.
      !!
      !! ** Method  :   Newtonian damping towards t_dta and s_dta computed 
      !!      and add to the general tracer trends:
      !!                     ta = ta + resto * (t_dta - tb)
      !!                     sa = sa + resto * (s_dta - sb)
      !!         The trend is computed either throughout the water column
      !!      (nlmdmp=0) or in area of weak vertical mixing (nlmdmp=1) or
      !!      below the well mixed layer (nlmdmp=2)
      !!
      !! ** Action  : - update the tracer trends (ta,sa) with the newtonian 
      !!                damping trends.
      !!              - save the trends in (ttrd,strd) ('key_trdtra')
      !! ASSUME key_zdfcst_tam
      !!----------------------------------------------------------------------
      !!
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index
      !!
      INTEGER  ::   ji, jj, jk            ! dummy loop indices
      REAL(wp) ::   ztest, ztaad, zsaad       ! temporary scalars
      !!----------------------------------------------------------------------
      IF( kt == nitend )   CALL tra_dmp_init_tam  ! Initialization

      ! 1. Newtonian damping trends on tracer fields
      ! --------------------------------------------
      !    compute the newtonian damping trends depending on nmldmp
      ztaad = 0.e0
      zsaad = 0.e0

      SELECT CASE ( nmldmp )
      !
      CASE( 0 )                ! newtonian damping throughout the water column
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ! add the trends to the general tracer trends
                  ztaad = ta_ad(ji,jj,jk) + ztaad
                  zsaad = sa_ad(ji,jj,jk) + zsaad

                  tb_ad(ji,jj,jk)  = tb_ad(ji,jj,jk) - ztaad * resto(ji,jj,jk)
                  sb_ad(ji,jj,jk)  = sb_ad(ji,jj,jk) - zsaad * resto(ji,jj,jk)
                  ztaad = 0.0_wp
                  zsaad = 0.0_wp
                END DO
            END DO
         END DO
         !
      CASE ( 1 )                ! no damping in the turbocline (avt > 5 cm2/s)
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ! add the trends to the general tracer trends
                  ztaad = ta_ad(ji,jj,jk) + ztaad
                  zsaad = sa_ad(ji,jj,jk) + zsaad
                  ztest = avt(ji,jj,jk) - 5.e-4
      !! ASSUME key_zdfcst_tam
                  IF( ztest < 0. ) THEN
                     tb_ad(ji,jj,jk)  = tb_ad(ji,jj,jk) - ztaad * resto(ji,jj,jk)
                     sb_ad(ji,jj,jk)  = sb_ad(ji,jj,jk) - zsaad * resto(ji,jj,jk)
                     ztaad = 0.0_wp
                     zsaad = 0.0_wp
                  ELSE
                     ztaad = 0.e0
                     zsaad = 0.e0
                  ENDIF
               END DO
            END DO
         END DO
         !
      CASE ( 2 )                ! no damping in the mixed layer 
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ! add the trends to the general tracer trends
                  ztaad = ta_ad(ji,jj,jk) + ztaad
                  zsaad = sa_ad(ji,jj,jk) + zsaad
                  IF( fsdept(ji,jj,jk) >= hmlp (ji,jj) ) THEN
                     tb_ad(ji,jj,jk)  = tb_ad(ji,jj,jk) - ztaad * resto(ji,jj,jk)
                     sb_ad(ji,jj,jk)  = sb_ad(ji,jj,jk) - zsaad * resto(ji,jj,jk)
                     ztaad = 0.0_wp
                     zsaad = 0.0_wp
                  ELSE
                     ztaad = 0.e0
                     zsaad = 0.e0
                  ENDIF
               END DO
            END DO
         END DO
         !
      END SELECT


   END SUBROUTINE tra_dmp_adj
   SUBROUTINE tra_dmp_init_tam
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE tra_dmp_init_tam  ***
      !! 
      !! ** Purpose :   Initialization for the newtonian damping 
      !!
      !! ** Method  :   read the nammbf namelist and check the parameters
      !!      called by tra_dmp at the first timestep (nit000)
      !!----------------------------------------------------------------------

      NAMELIST/namtdp/ ndmp, ndmpf, nmldmp, sdmp, bdmp, hdmp
      !!----------------------------------------------------------------------

      IF (lfirst) THEN

         REWIND ( numnam )                  ! Read Namelist namtdp : temperature and salinity damping term
         READ   ( numnam, namtdp )
         IF( lzoom )   nmldmp = 0           ! restoring to climatology at closed north or south boundaries
         
         IF(lwp) THEN                       ! Namelist print
            WRITE(numout,*)
            WRITE(numout,*) 'tra_dmp_tam : T and S newtonian damping'
            WRITE(numout,*) '~~~~~~~~~~~'
            WRITE(numout,*) '       Namelist namtdp : set damping parameter'
            WRITE(numout,*) '          T and S damping option         ndmp   = ', ndmp
            WRITE(numout,*) '          create a damping.coeff file    ndmpf  = ', ndmpf
            WRITE(numout,*) '          mixed layer damping option     nmldmp = ', nmldmp, '(zoom: forced to 0)'
            WRITE(numout,*) '          surface time scale (days)      sdmp   = ', sdmp
            WRITE(numout,*) '          bottom time scale (days)       bdmp   = ', bdmp
            WRITE(numout,*) '          depth of transition (meters)   hdmp   = ', hdmp
         ENDIF
         
         !! ** modified to allow damping at the equator
         
         SELECT CASE ( ndmp )
         CASE (  -1  )   ;   IF(lwp) WRITE(numout,*) '          tracer damping in the Med & Red seas only'
         CASE ( 0:90 )   ;   IF(lwp) WRITE(numout,*) '          tracer damping poleward of', ndmp, ' degrees'
         CASE DEFAULT
            IF(lwp) WRITE(numout,*) '          tracer damping disabled', ndmp
            !         WRITE(ctmp1,*) '          bad flag value for ndmp = ', ndmp
            !         CALL ctl_stop(ctmp1)
         END SELECT
         
         SELECT CASE ( nmldmp )
         CASE ( 0 )   ;   IF(lwp) WRITE(numout,*) '          tracer damping throughout the water column'
         CASE ( 1 )   ;   IF(lwp) WRITE(numout,*) '          no tracer damping in the turbocline (avt > 5 cm2/s)'
         CASE ( 2 )   ;   IF(lwp) WRITE(numout,*) '          no tracer damping in the mixed layer'
         CASE DEFAULT
            WRITE(ctmp1,*) '          bad flag value for nmldmp = ', nmldmp
            CALL ctl_stop(ctmp1)
         END SELECT
         
         IF( .NOT.lk_dtasal .OR. .NOT.lk_dtatem )   &
            &   CALL ctl_stop( 'no temperature and/or salinity data define key_dtatem and key_dtasal' )
         
         !                          ! Damping coefficients initialization
         IF( lzoom ) THEN   ;   CALL dtacof_zoom
         ELSE               ;   CALL dtacof
         ENDIF
         !
         lfirst = .FALSE.
      END IF
   END SUBROUTINE tra_dmp_init_tam

   SUBROUTINE tra_dmp_adj_tst ( kumadt ) 
      !!-----------------------------------------------------------------------
      !!
      !!          ***  ROUTINE tra_sbc_adj_tst : TEST OF tra_sbc_adj  ***
      !!
      !! ** Purpose : Test the adjoint routine.
      !!
      !! ** Method  : Verify the scalar product 
      !!           
      !!                 ( L dx )^T W dy  =  dx^T L^T W dy
      !!
      !!              where  L   = tangent routine
      !!                     L^T = adjoint routine
      !!                     W   = diagonal matrix of scale factors
      !!                     dx  = input perturbation (random field)
      !!                     dy  = L dx 
      !!
       !! History :
      !!        ! 08-08 (A. Vidard)
      !!-----------------------------------------------------------------------
      !! * Modules used

      !! * Arguments
      INTEGER, INTENT(IN) :: &
         & kumadt             ! Output unit
 
      INTEGER ::  &
         & istp,  &
         & jstp,  &
         & ji,    &        ! dummy loop indices
         & jj,    &        
         & jk     
      INTEGER, DIMENSION(jpi,jpj) :: &
         & iseed_2d        ! 2D seed for the random number generator

      !! * Local declarations
      REAL(KIND=wp), DIMENSION(:,:,:), ALLOCATABLE :: &
         & zsb_tlin,     &! Tangent input : before salinity
         & ztb_tlin,     &! Tangent input : before temperature
         & zsa_tlin,     &! Tangent input : after salinity
         & zta_tlin,     &! Tangent input : after temperature
         & zsa_tlout,    &! Tangent output: after salinity
         & zta_tlout,    &! Tangent output: after temperature
         & zsb_adout,    &! Adjoint output : before salinity
         & ztb_adout,    &! Adjoint output : before temperature
         & zsa_adout,    &! Adjoint output : after salinity
         & zta_adout,    &! Adjoint output : after temperature
         & zsa_adin,     &! Adjoint input : after salinity
         & zta_adin,     &! Adjoint input : after temperature
         & z3r            ! 3D field
         
      REAL(KIND=wp) ::       &
         & zsp1,             & ! scalar product involving the tangent routine
         & zsp1_1,           & ! scalar product involving the tangent routine
         & zsp1_2,           & ! scalar product involving the tangent routine
         & zsp1_3,           & ! scalar product involving the tangent routine
         & zsp1_4,           & ! scalar product involving the tangent routine
         & zsp2,             & ! scalar product involving the adjoint routine
         & zsp2_1,           & ! scalar product involving the adjoint routine
         & zsp2_2,           & ! scalar product involving the adjoint routine
         & zsp2_3,           & ! scalar product involving the adjoint routine
         & zsp2_4              ! scalar product involving the adjoint routine
      CHARACTER(LEN=14) ::   &
         & cl_name

      ALLOCATE( & 
         & ztb_tlin(jpi,jpj,jpk),     &
         & zsb_tlin(jpi,jpj,jpk),     &
         & zsa_tlin(jpi,jpj,jpk),     &
         & zta_tlin(jpi,jpj,jpk),     &
         & zsa_tlout(jpi,jpj,jpk),    &
         & zta_tlout(jpi,jpj,jpk),    &
         & ztb_adout(jpi,jpj,jpk),    &
         & zsb_adout(jpi,jpj,jpk),    &
         & zsa_adout(jpi,jpj,jpk),    &
         & zta_adout(jpi,jpj,jpk),    &
         & zsa_adin(jpi,jpj,jpk),     &
         & zta_adin(jpi,jpj,jpk),     &
         & z3r         (jpi,jpj,jpk)  &
         & )
      ! Test for time steps nit000 and nit000 + 1 (the matrix changes)

      DO jstp = nit000, nit000 + 2
         istp = jstp
         IF ( jstp == nit000 +2 ) istp = nitend

      ! Initialize the reference state
      avt(:,:,:) = 1.e-1
      !=============================================================
      ! 1) dx = ( T ) and dy = ( T )
      !=============================================================

      !--------------------------------------------------------------------
      ! Reset the tangent and adjoint variables
      !--------------------------------------------------------------------
      zsb_tlin(:,:,:)      = 0.0_wp 
      ztb_tlin(:,:,:)      = 0.0_wp 
      zsa_tlin(:,:,:)      = 0.0_wp 
      zta_tlin(:,:,:)      = 0.0_wp 
      zsa_tlout(:,:,:)     = 0.0_wp 
      zta_tlout(:,:,:)     = 0.0_wp 
      zsb_adout(:,:,:)     = 0.0_wp 
      ztb_adout(:,:,:)     = 0.0_wp 
      zsa_adout(:,:,:)     = 0.0_wp 
      zta_adout(:,:,:)     = 0.0_wp 
      zsa_adin(:,:,:)      = 0.0_wp 
      zta_adin(:,:,:)      = 0.0_wp 

      sa_ad(:,:,:)      = 0.0_wp 
      sb_ad(:,:,:)      = 0.0_wp 
      ta_ad(:,:,:)      = 0.0_wp 
      tb_ad(:,:,:)      = 0.0_wp 
      sa_tl(:,:,:)      = 0.0_wp 
      sb_tl(:,:,:)      = 0.0_wp 
      ta_tl(:,:,:)      = 0.0_wp 
      tb_tl(:,:,:)      = 0.0_wp 

      DO jj = 1, jpj
         DO ji = 1, jpi
            iseed_2d(ji,jj) = - ( 785483 + &
               &                  mig(ji) + ( mjg(jj) - 1 ) * jpiglo )
         END DO
      END DO
      CALL grid_random( iseed_2d, z3r, 'T', 0.0_wp, stds )
      DO jk = 1, jpk
         DO jj = nldj, nlej
            DO ji = nldi, nlei
               zsb_tlin(ji,jj,jk) = z3r(ji,jj,jk) 
            END DO
         END DO
      END DO
      DO jj = 1, jpj
         DO ji = 1, jpi
            iseed_2d(ji,jj) = - ( 358606 + &
               &                  mig(ji) + ( mjg(jj) - 1 ) * jpiglo )
         END DO
      END DO
      CALL grid_random( iseed_2d, z3r, 'T', 0.0_wp, stdt )
      DO jk = 1, jpk
         DO jj = nldj, nlej
            DO ji = nldi, nlei
               ztb_tlin(ji,jj,jk) = z3r(ji,jj,jk) 
            END DO
         END DO
      END DO

      DO jj = 1, jpj
         DO ji = 1, jpi
            iseed_2d(ji,jj) = - ( 596035 + &
               &                  mig(ji) + ( mjg(jj) - 1 ) * jpiglo )
         END DO
      END DO
      CALL grid_random( iseed_2d, z3r, 'T', 0.0_wp, stds )
      DO jk = 1, jpk
         DO jj = nldj, nlej
            DO ji = nldi, nlei
               zsa_tlin(ji,jj,jk) = z3r(ji,jj,jk) 
            END DO
         END DO
      END DO

      DO jj = 1, jpj
         DO ji = 1, jpi
            iseed_2d(ji,jj) = - ( 523432 + &
               &                  mig(ji) + ( mjg(jj) - 1 ) * jpiglo )
         END DO
      END DO
      CALL grid_random( iseed_2d, z3r, 'T', 0.0_wp, stdt )
      DO jk = 1, jpk
         DO jj = nldj, nlej
            DO ji = nldi, nlei
               zta_tlin(ji,jj,jk) = z3r(ji,jj,jk) 
            END DO
         END DO
      END DO

      sb_tl(:,:,:) = zsb_tlin(:,:,:) 
      tb_tl(:,:,:) = ztb_tlin(:,:,:) 
      sa_tl(:,:,:) = zsa_tlin(:,:,:) 
      ta_tl(:,:,:) = zta_tlin(:,:,:)
     
      CALL tra_dmp_tan( istp )

      zsa_tlout(:,:,:)     = sa_tl(:,:,:) 
      zta_tlout(:,:,:)     = ta_tl(:,:,:)

      !--------------------------------------------------------------------
      ! Initialize the adjoint variables: dy^* = W dy
      !--------------------------------------------------------------------

      DO jk = 1, jpk
        DO jj = nldj, nlej
           DO ji = nldi, nlei
              zsa_adin(ji,jj,jk)     = zsa_tlout(ji,jj,jk) &
                 &                   * e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) &
                 &                   * tmask(ji,jj,jk) * wesp_s(jk)
              zta_adin(ji,jj,jk)     = zta_tlout(ji,jj,jk) &
                 &                   * e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) &
                 &                   * tmask(ji,jj,jk) * wesp_t(jk)
            END DO
         END DO
      END DO

      !--------------------------------------------------------------------
      ! Compute the scalar product: ( L dx )^T W dy
      !--------------------------------------------------------------------

      zsp1_1 = DOT_PRODUCT( zsa_tlout    , zsa_adin     )
      zsp1_2 = DOT_PRODUCT( zta_tlout    , zta_adin     )
      zsp1   = zsp1_1 + zsp1_2 
      !--------------------------------------------------------------------
      ! Call the adjoint routine: dx^* = L^T dy^*
      !--------------------------------------------------------------------

      sa_ad(:,:,:)     = zsa_adin(:,:,:)
      ta_ad(:,:,:)     = zta_adin(:,:,:)

      CALL tra_dmp_adj( istp )

      zsb_adout(:,:,:) = sb_ad(:,:,:)
      ztb_adout(:,:,:) = tb_ad(:,:,:)
      zsa_adout(:,:,:) = sa_ad(:,:,:)
      zta_adout(:,:,:) = ta_ad(:,:,:)

      !--------------------------------------------------------------------
      ! Compute the scalar product: dx^T L^T W dy
      !--------------------------------------------------------------------

      zsp2_1 = DOT_PRODUCT( zsb_tlin  , zsb_adout   )
      zsp2_2 = DOT_PRODUCT( ztb_tlin  , ztb_adout   )
      zsp2_3 = DOT_PRODUCT( zsa_tlin  , zsa_adout   )
      zsp2_4 = DOT_PRODUCT( zta_tlin  , zta_adout   )

      zsp2 = zsp2_1 + zsp2_2 + zsp2_3 + zsp2_4

      ! Compare the scalar products

      ! 14 char:'12345678901234'
      IF ( istp == nit000 ) THEN
         cl_name = 'tra_dmp_adj T1'
      ELSEIF ( istp == nit000 +1 ) THEN
         cl_name = 'tra_dmp_adj T2'
      ELSEIF ( istp == nitend ) THEN
         cl_name = 'tra_dmp_adj T3'
      END IF
      CALL prntst_adj( cl_name, kumadt, zsp1, zsp2 )

      END DO

      DEALLOCATE( & 
         & zsb_tlin,     &
         & ztb_tlin,     &
         & zsa_tlin,     &
         & zta_tlin,     &
         & zsa_tlout,    &
         & zta_tlout,    &
         & zsb_adout,    &
         & ztb_adout,    &
         & zsa_adout,    &
         & zta_adout,    &
         & zsa_adin,     &
         & zta_adin,     &
         & z3r           &
         & )

   END SUBROUTINE tra_dmp_adj_tst

#else
   !!----------------------------------------------------------------------
   !!   Default key                                     NO internal damping
   !!----------------------------------------------------------------------
   LOGICAL , PUBLIC, PARAMETER ::   lk_tradmp = .FALSE.    !: internal damping flag
CONTAINS
   SUBROUTINE tra_dmp_tan( kt )        ! Empty routine
      WRITE(*,*) 'tra_dmp_tan: You should not have seen this print! error?', kt
   END SUBROUTINE tra_dmp_tan
   SUBROUTINE tra_dmp_adj( kt )        ! Empty routine
      WRITE(*,*) 'tra_dmp_adj: You should not have seen this print! error?', kt
   END SUBROUTINE tra_dmp_adj
   SUBROUTINE tra_dmp_adj_tst( kt )        ! Empty routine
      WRITE(*,*) 'tra_dmp_adj_tst: You should not have seen this print! error?', kt
   END SUBROUTINE tra_dmp_adj_tst
#endif

   !!======================================================================
#endif
END MODULE tradmp_tam
