MODULE trj_tam
#ifdef key_tam
   !!======================================================================
   !!                       ***  MODULE trj_tam ***
   !! NEMOVAR trajectory: Allocate and read the trajectory for linearzation
   !!======================================================================

   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   bkg_init : Initialize the background fields from disk
   !!----------------------------------------------------------------------
   !! * Modules used   
   USE par_kind, ONLY : & ! Precision variables
      & wp
   USE par_oce, ONLY : &  ! Ocean space and time domain variables
      & jpi, jpj, jpk
   USE tamtrj             ! Parameters for the assmilation interface
   USE in_out_manager, ONLY : & ! I/O manager
      & lwp 
   USE oce                ! Model variables 
   USE zdf_oce            ! Vertical mixing variables
   USE zdfddm             ! Double diffusion mixing parameterization
   USE ldftra_oce         ! Lateral tracer mixing coefficient defined in memory
   USE ldfslp             ! Slopes of neutral surfaces
   USE tradmp             ! Tracer damping
   USE sbc_oce            ! Ocean surface boundary conditions
   USE iom                ! Library to read input files

   USE zdfmxl, ONLY : &   ! mixed layer depth
      & hmlp
   USE divcur             ! horizontal divergence and relative vorticity
   USE wzvmod             !  vertical velocity
  
   IMPLICIT NONE

   !! * Routine accessibility
   PRIVATE
   PUBLIC &
      & trj_rea,     &   !: Read trajectory at time step kstep into now fields
      & trj_rd_spl,  &   !: Read simple data (without interpolation)
      & trj_wri_spl      !: Write simple data (without interpolation)

   !! * Module variables
   INTEGER, SAVE :: inumtrj1 = -1, inumtrj2 = -1
   REAL(wp), SAVE :: &
      & stpr1, &
      & stpr2
   REAL(wp), ALLOCATABLE, DIMENSION(:,:), SAVE :: &
      & empr1,    &
      & empsr1,   &   
      & empr2,    &
      & empsr2
#if defined key_traldf_eiv
#if defined key_traldf_c3d
   REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: &
#elif defined key_traldf_c2d
   REAL(wp), ALLOCATABLE, DIMENSION(:,:), SAVE :: &
#elif defined key_traldf_c1d
   REAL(wp), ALLOCATABLE, DIMENSION(:), SAVE :: &
#else
   REAL(wp) ::
#endif
      & aeiur1,   &
      & aeivr1,   &
      & aeiwr1,   & 
      & aeiur2,   &
      & aeivr2,   &
      & aeiwr2
#endif
  REAL(wp), ALLOCATABLE, DIMENSION(:,:,:), SAVE :: &
      & unr1,     &
      & vnr1,     &
      & tnr1,     &
      & snr1,     &
      & avmur1,   &
      & avmvr1,   &
      & avtr1,    &
      & uslpr1,   &
      & vslpr1,   &
      & wslpir1,  &
      & wslpjr1,  &
      & avsr1,    &
      & tar1,     &
      & sar1,     &
      & tbr1,     &
      & sbr1,     &
      & unr2,     &
      & vnr2,     &
      & tnr2,     &
      & snr2,     &
      & avmur2,   &
      & avmvr2,   &
      & avtr2,    &
      & uslpr2,   &
      & vslpr2,   &
      & wslpir2,  &
      & wslpjr2,  &
      & avsr2,    &
      & tar2,     &
      & sar2,     &
      & tbr2,     &
      & sbr2
  REAL(wp), ALLOCATABLE, DIMENSION(:,:), SAVE :: &
      & hmlp1,    &
      & hmlp2  
      
CONTAINS

   SUBROUTINE trj_rea( kstp, kdir )
      !!-----------------------------------------------------------------------
      !!
      !!                  ***  ROUTINE trj_reat  ***
      !!
      !! ** Purpose : Read from file the trjectory from the outer loop
      !!
      !! ** Method  : IOM
      !!
      !! ** Action  :
      !!                   
      !! References : 
      !!
      !! History :
      !!        ! 08-05 (K. Mogensen) Initial version
      !!        ! 09-03 (F.Vigilant) Add reading of hmlp and calls (divcur, wzvmod)
      !!-----------------------------------------------------------------------
      !! * Modules used
      !! * Arguments
      INTEGER, INTENT(in) :: &
         & kstp, &           ! Step for requested trajectory
         & kdir              ! Direction for stepping (1 forward, -1 backward)
      !! * Local declarations
      CHARACTER (LEN=100) :: &
         & cl_asmtrj
      INTEGER :: &
         & inrcm,  &
         & inrcp,  &
         & inrc,   &
         & istpr1, &
         & istpr2, &
	 & it
      REAL(KIND=wp) :: &
         & zwtr1, &
         & zwtr2, &
         & zden,  &
         & zstp

      ! Initialize data and open file
      !! if step time is corresponding to a saved state
      IF ( ( MOD( kstp - nit000 + 1, nittrjfrq ) == 0 )  ) THEN       

         it = kstp - nit000 + 1

         IF ( inumtrj1 == -1 ) THEN

            ! Define the input file 
            WRITE(cl_asmtrj, FMT='(A,A,I5.5,".nc")' ) TRIM( c_asmtrj ), '_', it

            !         WRITE(cl_asmtrj, FMT='(A,".nc")' ) TRIM( c_asmtrj )
            cl_asmtrj = TRIM( cl_asmtrj )

            IF(lwp) THEN

               WRITE(numout,*)
               WRITE(numout,*)'Reading non-linear fields from : ',TRIM(cl_asmtrj)
               WRITE(numout,*)

            ENDIF
            CALL iom_open( cl_asmtrj, inumtrj1 )     
            if ( inumtrj1 == -1)  CALL ctl_stop( 'No tam_trajectory cl_amstrj found' )
   
            ALLOCATE( &
               & empr1(jpi,jpj),  &
               & empsr1(jpi,jpj), &
               & empr2(jpi,jpj),  &
               & empsr2(jpi,jpj)  &
               & )

            ALLOCATE( &
               & unr1(jpi,jpj,jpk),     &
               & vnr1(jpi,jpj,jpk),     &
               & tnr1(jpi,jpj,jpk),     &
               & snr1(jpi,jpj,jpk),     &
               & avmur1(jpi,jpj,jpk),   &
               & avmvr1(jpi,jpj,jpk),   &
               & avtr1(jpi,jpj,jpk),    &
               & tar1(jpi,jpj,jpk),     &
               & sar1(jpi,jpj,jpk),     &
               & tbr1(jpi,jpj,jpk),     &
               & sbr1(jpi,jpj,jpk),     &
               & unr2(jpi,jpj,jpk),     &
               & vnr2(jpi,jpj,jpk),     &
               & tnr2(jpi,jpj,jpk),     &
               & snr2(jpi,jpj,jpk),     &
               & avmur2(jpi,jpj,jpk),   &
               & avmvr2(jpi,jpj,jpk),   &
               & avtr2(jpi,jpj,jpk),    &
               & tar2(jpi,jpj,jpk),     &
               & sar2(jpi,jpj,jpk),     &
               & tbr2(jpi,jpj,jpk),     &
               & sbr2(jpi,jpj,jpk)      &
               & )

#if defined key_traldf_eiv
#if defined key_traldf_c3d
#elif defined key_traldf_c2d
            ALLOCATE( &
               & aeiur1(jpi,jpj), &
               & aeivr1(jpi,jpj), &
               & aeiwr1(jpi,jpj), &
               & aeiur2(jpi,jpj), &
               & aeivr2(jpi,jpj), &
               & aeiwr2(jpi,jpj)  &
               & )
#elif defined key_traldf_c1d
#endif
#endif

#if defined key_ldfslp
            ALLOCATE( &
               & uslpr1(jpi,jpj,jpk),   &
               & vslpr1(jpi,jpj,jpk),   &
               & wslpir1(jpi,jpj,jpk),  &
               & wslpjr1(jpi,jpj,jpk),  &
               & uslpr2(jpi,jpj,jpk),   &
               & vslpr2(jpi,jpj,jpk),   &
               & wslpir2(jpi,jpj,jpk),  &
               & wslpjr2(jpi,jpj,jpk)   &
               & )
#endif

#if defined key_zdfddm
            ALLOCATE( &
               & avsr1(jpi,jpj,jpk),    &
               & avsr2(jpi,jpj,jpk)     &
               & )
#endif

#if defined key_tradmp
            ALLOCATE( &
               & hmlp1(jpi,jpj),    &
               & hmlp2(jpi,jpj)     &
               & )
#endif

         ENDIF


      ! Read records

         inrcm = INT( ( kstp - nit000 + 1 ) / nittrjfrq ) + 1

         ! Copy record 1 into record 2

         IF ( ( kstp /= nitend )         .AND. &
            & ( kstp - nit000 + 1 /= 0 ) .AND. &
            & ( kdir == -1 ) ) THEN

            stpr2           = stpr1

            empr2   (:,:)   = empr1   (:,:)
            empsr2  (:,:)   = empsr1  (:,:)

            unr2    (:,:,:) = unr1    (:,:,:)
            vnr2    (:,:,:) = vnr1    (:,:,:)
            tnr2    (:,:,:) = tnr1    (:,:,:)
            snr2    (:,:,:) = snr1    (:,:,:)
            avmur2  (:,:,:) = avmur1  (:,:,:)
            avmvr2  (:,:,:) = avmvr1  (:,:,:)
            avtr2   (:,:,:) = avtr1   (:,:,:)
#if defined key_ldfslp
            uslpr2  (:,:,:) = uslpr1  (:,:,:)
            vslpr2  (:,:,:) = vslpr1  (:,:,:)
            wslpir2 (:,:,:) = wslpir1 (:,:,:)
            wslpjr2 (:,:,:) = wslpjr1 (:,:,:)
#endif
#if defined key_zdfddm
            avsr2   (:,:,:) = avsr1   (:,:,:)
#endif
            tar2    (:,:,:) = tar1    (:,:,:)
            sar2    (:,:,:) = sar1    (:,:,:)
            tbr2    (:,:,:) = tbr1    (:,:,:)
            sbr2    (:,:,:) = sbr1    (:,:,:)
#if defined key_tradmp
            hmlp1   (:,:)   = hmlp2   (:,:)
#endif
#if defined key_traldf_eiv  
#if defined key_traldf_c3d
            aeiur2  (:,:,:) = aeiur1  (:,:,:)
            aeivr2  (:,:,:) = aeivr1  (:,:.:)
            aeiwr2  (:,:,:) = aeiwr1  (:,:.:)
#elif defined key_traldf_c2d
            aeiur2  (:,:)   = aeiur1  (:,:)
            aeivr2  (:,:)   = aeivr1  (:,:)
            aeiwr2  (:,:)   = aeiwr1  (:,:)
#elif defined key_traldf_c1d
            aeiur2  (:)     = aeiur1  (:)
            aeivr2  (:)     = aeivr1  (:)
            aeiwr2  (:)     = aeiwr1  (:)
#else
            aeiur2          = aeiur1     
            aeivr2          = aeivr1     
            aeiwr2          = aeiwr1     
#endif
#endif

            istpr1 = INT( stpr1 )

            IF(lwp) WRITE(numout,*) &
               &                 '    Trajectory record copy time step = ', istpr1

         ENDIF
! added
         IF ( ( kstp - nit000 + 1 /= 0 ) .AND. ( kdir == -1 ) ) THEN
            ! We update the input filename 
            WRITE(cl_asmtrj, FMT='(A,A,I5.5,".nc")' ) TRIM( c_asmtrj ), '_', (it-nittrjfrq)
            cl_asmtrj = TRIM( cl_asmtrj )
            IF(lwp) THEN
               WRITE(numout,*)
               WRITE(numout,*)'Reading non-linear fields from : ',TRIM(cl_asmtrj)
               WRITE(numout,*)
            ENDIF
         ENDIF
! end added
         ! Read record 1

         IF ( ( kstp - nit000 + 1 == 0 ) .AND.( kdir == 1           ) .OR. &
            & ( kstp - nit000 + 1 /= 0 ) .AND.( kdir == -1          ) ) THEN

            IF ( kdir == -1 ) inrcm = inrcm - 1
!added
!            inrc = inrcm
            ! temporary fix: currently, only one field by step time
            inrc = 1
            stpr1 = (inrcm - 1) * nittrjfrq
!            stpr1 = (inrc - 1) * nittrjfrq 
!end added

            ! bug fixed to read several time the initial data
            IF ( ( kstp - nit000 + 1 == 0 ) .AND. ( kdir == 1 ) ) THEN
               ! Define the input file 
               WRITE(cl_asmtrj, FMT='(A,A,I5.5,".nc")' ) TRIM( c_asmtrj ), '_', it

               cl_asmtrj = TRIM( cl_asmtrj )

               IF(lwp) THEN
                  WRITE(numout,*)
                  WRITE(numout,*)'Reading non-linear fields from : ',TRIM(cl_asmtrj)
                  WRITE(numout,*)
               ENDIF
            END IF
            IF ( inumtrj1 /= -1 )   CALL iom_open( cl_asmtrj, inumtrj1 )

            CALL iom_get( inumtrj1, jpdom_data, 'emp'   , empr1   , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'emps'  , empsr1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'un'    , unr1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'vn'    , vnr1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'tn'    , tnr1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'sn'    , snr1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'avmu'  , avmur1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'avmv'  , avmvr1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'avt'   , avtr1   , inrc )
#if defined key_ldfslp
            CALL iom_get( inumtrj1, jpdom_data, 'uslp'  , uslpr1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'vslp'  , vslpr1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'wslpi' , wslpir1 , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'wslpj' , wslpjr1 , inrc )
#endif
#if defined key_zdfddm
            CALL iom_get( inumtrj1, jpdom_data, 'avs'   , avsr1   , inrc )
#endif
            CALL iom_get( inumtrj1, jpdom_data, 'ta'    , tar1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'sa'    , sar1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'tb'    , tbr1    , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'sb'    , sbr1    , inrc )
#if defined key_tradmp
            CALL iom_get( inumtrj1, jpdom_data, 'hmlp'  , hmlp1   , inrc )
#endif
#if defined key_traldf_eiv
            CALL iom_get( inumtrj1, jpdom_data, 'aeiu'  , aeiur1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'aeiv'  , aeivr1  , inrc )
            CALL iom_get( inumtrj1, jpdom_data, 'aeiw'  , aeiwr1  , inrc )
#endif
            CALL iom_close( inumtrj1 )

            istpr1 = INT( stpr1 )
            IF(lwp)WRITE(numout,*) '   trajectory read time step = ', istpr1,&
               &                   '  record = ', inrc

         ENDIF


         ! Copy record 2 into record 1

         IF ( ( kstp - nit000 + 1 /= 0 ) .AND. &
            & ( kstp /= nitend         ) .AND. &
            & ( kdir == 1              ) ) THEN

            stpr1           = stpr2
            empr1   (:,:)   = empr2   (:,:)
            empsr1  (:,:)   = empsr2  (:,:)
            unr1    (:,:,:) = unr2    (:,:,:)
            vnr1    (:,:,:) = vnr2    (:,:,:)
            tnr1    (:,:,:) = tnr2    (:,:,:)
            snr1    (:,:,:) = snr2    (:,:,:)
            avmur1  (:,:,:) = avmur2  (:,:,:)
            avmvr1  (:,:,:) = avmvr2  (:,:,:)
            avtr1   (:,:,:) = avtr2   (:,:,:)
#if defined key_ldfslp
            uslpr1  (:,:,:) = uslpr2  (:,:,:)
            vslpr1  (:,:,:) = vslpr2  (:,:,:)
            wslpir1 (:,:,:) = wslpir2 (:,:,:)
            wslpjr1 (:,:,:) = wslpjr2 (:,:,:)
#endif
#if defined key_zdfddm
            avsr1   (:,:,:) = avsr2   (:,:,:)
#endif
            tar1    (:,:,:) = tar2    (:,:,:)
            sar1    (:,:,:) = sar2    (:,:,:)
            tbr1    (:,:,:) = tbr2    (:,:,:)
            sbr1    (:,:,:) = sbr2    (:,:,:)
#if defined key_tradmp
            hmlp1   (:,:)   = hmlp2   (:,:)
#endif
#if defined key_traldf_eiv  
#if defined key_traldf_c3d
            aeiur1  (:,:,:) = aeiur2  (:,:,:)
            aeivr1  (:,:,:) = aeivr2  (:,:.:)
            aeiwr1  (:,:,:) = aeiwr2  (:,:.:)
#elif defined key_traldf_c2d
            aeiur1  (:,:)   = aeiur2  (:,:)
            aeivr1  (:,:)   = aeivr2  (:,:)
            aeiwr1  (:,:)   = aeiwr2  (:,:)
#elif defined key_traldf_c1d
            aeiur1  (:)     = aeiur2  (:)
            aeivr1  (:)     = aeivr2  (:)
            aeiwr1  (:)     = aeiwr2  (:)
#else
            aeiur1          = aeiur2     
            aeivr1          = aeivr2     
            aeiwr1          = aeiwr2     
#endif
#endif

            istpr1 = INT( stpr1 )
            IF(lwp) WRITE(numout,*) &
               &                 '   Trajectory record copy time step = ', istpr1

         ENDIF

         ! Read record 2

!!         IF ( ( kstp /= nitend ) .AND. ( kdir == 1  ) .OR. &
!!            & ( kstp == nitend ) .AND. ( kdir == -1 ) ) THEN    
! change
         IF ( ( ( kstp /= nitend ) .AND. ( kdir == 1  )) .OR. &
            &   ( kstp == nitend ) .AND.(  kdir == -1   ) ) THEN
! end change
!added
!            ! Need to open next saved file when kstp = initial step   
!            IF  ( kstp - nit000 + 1 == 0 ) THEN     
            ! Need to open next saved file when kstp = initial step   
! change     
!            IF ( ( kstp /= nitend ) .AND. ( kdir == 1 ) ) THEN 
! end change
! end added
               ! Define the input file 
               IF  (  kdir == -1   ) THEN
                   WRITE(cl_asmtrj, FMT='(A,A,I5.5,".nc")' ) TRIM( c_asmtrj ), '_', (it)
               ELSE      
                  WRITE(cl_asmtrj, FMT='(A,A,I5.5,".nc")' ) TRIM( c_asmtrj ), '_', (it+nittrjfrq)
               ENDIF
               cl_asmtrj = TRIM( cl_asmtrj )

               IF(lwp) THEN        
                  WRITE(numout,*)
                  WRITE(numout,*)'Reading non-linear fields from : ',TRIM(cl_asmtrj)
                  WRITE(numout,*)        
               ENDIF

               CALL iom_open( cl_asmtrj, inumtrj2 ) 
! change
!            END IF
!end change

            inrcp = inrcm + 1
            !            inrc  = inrcp
!added
            inrc = 1  ! temporary  fix
!end added
            stpr2 = (inrcp - 1) * nittrjfrq 
            CALL iom_get( inumtrj2, jpdom_data, 'emp'   , empr2   , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'emps'  , empsr2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'un'    , unr2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'vn'    , vnr2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'tn'    , tnr2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'sn'    , snr2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'avmu'  , avmur2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'avmv'  , avmvr2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'avt'   , avtr2   , inrc )
#if defined key_ldfslp
            CALL iom_get( inumtrj2, jpdom_data, 'uslp'  , uslpr2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'vslp'  , vslpr2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'wslpi' , wslpir2 , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'wslpj' , wslpjr2 , inrc )
#endif
#if defined key_zdfddm
            CALL iom_get( inumtrj2, jpdom_data, 'avs'   , avsr2   , inrc )
#endif
            CALL iom_get( inumtrj2, jpdom_data, 'ta'    , tar2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'sa'    , sar2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'tb'    , tbr2    , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'sb'    , sbr2    , inrc )
#if defined key_tradmp
            CALL iom_get( inumtrj2, jpdom_data, 'hmlp'  , hmlp2   , inrc )
#endif
#if defined key_traldf_eiv
            CALL iom_get( inumtrj2, jpdom_data, 'aeiu'  , aeiur2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'aeiv'  , aeivr2  , inrc )
            CALL iom_get( inumtrj2, jpdom_data, 'aeiw'  , aeiwr2  , inrc )
#endif
            CALL iom_close( inumtrj2 )

            istpr2 = INT( stpr2 )
            IF(lwp)WRITE(numout,*) '   trajectory read2 time step = ', istpr2,&
               &                   '  record = ', inrc
         ENDIF

      ENDIF

      ! Linear interpolate to the current step

      IF(lwp)WRITE(numout,*) '   linear interpolate to current', &
         &                   ' time step = ', kstp

      ! Interpolation coefficients

      zstp = kstp - nit000 + 1
      zden   = 1.0 / ( stpr2 - stpr1 )

      zwtr1  = ( stpr2 - zstp      ) * zden
      zwtr2  = ( zstp  - stpr1     ) * zden

IF(lwp)WRITE(numout,*) '   linear interpolate coeff.', &
         &                   '  = ', zwtr1, zwtr2

      emp(:,:)      = zwtr1 * empr1   (:,:)   + zwtr2 * empr2   (:,:)
      emps(:,:)     = zwtr1 * empsr1  (:,:)   + zwtr2 * empsr2  (:,:)
      un(:,:,:)     = zwtr1 * unr1    (:,:,:) + zwtr2 * unr2    (:,:,:)
      vn(:,:,:)     = zwtr1 * vnr1    (:,:,:) + zwtr2 * vnr2    (:,:,:)
      tn(:,:,:)     = zwtr1 * tnr1    (:,:,:) + zwtr2 * tnr2    (:,:,:)
      sn(:,:,:)     = zwtr1 * snr1    (:,:,:) + zwtr2 * snr2    (:,:,:)
      avmu(:,:,:)   = zwtr1 * avmur1  (:,:,:) + zwtr2 * avmur2  (:,:,:)
      avmv(:,:,:)   = zwtr1 * avmvr1  (:,:,:) + zwtr2 * avmvr2  (:,:,:)
      avt(:,:,:)    = zwtr1 * avtr1   (:,:,:) + zwtr2 * avtr2   (:,:,:)
#if defined key_ldfslp
      uslp(:,:,:)   = zwtr1 * uslpr1  (:,:,:) + zwtr2 * uslpr2  (:,:,:)
      vslp(:,:,:)   = zwtr1 * vslpr1  (:,:,:) + zwtr2 * vslpr2  (:,:,:)
      wslpi(:,:,:)  = zwtr1 * wslpir1 (:,:,:) + zwtr2 * wslpir2 (:,:,:)
      wslpj(:,:,:)  = zwtr1 * wslpjr1 (:,:,:) + zwtr2 * wslpjr2 (:,:,:)
#endif
#if defined key_zdfddm
      avs(:,:,:)    = zwtr1 * avsr1   (:,:,:) + zwtr2 * avsr2   (:,:,:)
#endif
      ta(:,:,:)     = zwtr1 * tar1    (:,:,:) + zwtr2 * tar2    (:,:,:)
      sa(:,:,:)     = zwtr1 * sar1    (:,:,:) + zwtr2 * sar2    (:,:,:)
      tb(:,:,:)     = zwtr1 * tbr1    (:,:,:) + zwtr2 * tbr2    (:,:,:)
      sb(:,:,:)     = zwtr1 * sbr1    (:,:,:) + zwtr2 * sbr2    (:,:,:)
#if defined key_tradmp
      hmlp(:,:)     = zwtr1 * hmlp1(:,:)    + zwtr2 * hmlp2(:,:)
#endif
#if defined key_traldf_eiv  
#if defined key_traldf_c3d
      aeiu(:,:,:)   = zwtr1 * aeiur1  (:,:,:) + zwtr2 * aeiur2  (:,:,:)
      aeiv(:,:,:)   = zwtr1 * aeivr1  (:,:,:) + zwtr2 * aeivr2  (:,:.:)
      aeiw(:,:,:)   = zwtr1 * aeiwr1  (:,:,:) + zwtr2 * aeiwr2  (:,:.:)
#elif defined key_traldf_c2d
      aeiu(:,:)     = zwtr1 * aeiur1  (:,:)   + zwtr2 * aeiur2  (:,:)
      aeiv(:,:)     = zwtr1 * aeivr1  (:,:)   + zwtr2 * aeivr2  (:,:)
      aeiw(:,:)     = zwtr1 * aeiwr1  (:,:)   + zwtr2 * aeiwr2  (:,:)
#elif defined key_traldf_c1d
      aeiu(:)       = zwtr1 * aeiur1  (:)     + zwtr2 * aeiur2  (:)
      aeiv(:)       = zwtr1 * aeivr1  (:)     + zwtr2 * aeivr2  (:)
      aeiw(:)       = zwtr1 * aeiwr1  (:)     + zwtr2 * aeiwr2  (:)
#else
      aeiu          = zwtr1 * aeiur1          + zwtr2 * aeiur2     
      aeiv          = zwtr1 * aeivr1          + zwtr2 * aeivr2     
      aeiw          = zwtr1 * aeiwr1          + zwtr2 * aeiwr2     
#endif
#endif

      CALL div_cur(kstp)
      CALL wzv(kstp)

   END SUBROUTINE trj_rea


   SUBROUTINE trj_wri_spl(filename)
      !!-----------------------------------------------------------------------
      !!
      !!                  ***  ROUTINE trj_wri_spl ***
      !!
      !! ** Purpose : Write SimPLe data to file the model state trajectory
      !!
      !! ** Method  : 
      !!
      !! ** Action  :
      !!
      !! History : 
      !!        ! 09-07 (F. Vigilant) 
      !!-----------------------------------------------------------------------
      !! *Module udes
      USE iom  
      !! * Arguments
      !! * Local declarations
      INTEGER :: &
         & inum, &                  ! File unit number
         & fd                       ! field number
      CHARACTER (LEN=50) :: &
         & filename      

      fd=1
      WRITE(filename, FMT='(A,A)' ) TRIM( filename ), '.nc'
      filename = TRIM( filename )
      CALL iom_open( filename, inum, ldwrt = .TRUE., kiolib = jprstlib)

      ! Output trajectory fields
      CALL iom_rstput( fd, fd, inum, 'un'   , un   )
      CALL iom_rstput( fd, fd, inum, 'vn'   , vn   )
      CALL iom_rstput( fd, fd, inum, 'tn'   , tn   )
      CALL iom_rstput( fd, fd, inum, 'sn'   , sn   )
      CALL iom_rstput( fd, fd, inum, 'sshn' , sshn )
      CALL iom_rstput( fd, fd, inum, 'wn'   , wn   )
      CALL iom_rstput( fd, fd, inum, 'tb'   , tb   )
      CALL iom_rstput( fd, fd, inum, 'sb'   , sb   )
      CALL iom_rstput( fd, fd, inum, 'ua'   , ua   )
      CALL iom_rstput( fd, fd, inum, 'va'   , va   )
      CALL iom_rstput( fd, fd, inum, 'ta'   , ta   )
      CALL iom_rstput( fd, fd, inum, 'sa'   , sa   )
      CALL iom_rstput( fd, fd, inum, 'sshb' , sshb )
      CALL iom_rstput( fd, fd, inum, 'rhd'  , rhd  )
      CALL iom_rstput( fd, fd, inum, 'rhop' , rhop )
      CALL iom_rstput( fd, fd, inum, 'gtu'  , gtu  )
      CALL iom_rstput( fd, fd, inum, 'gsu'  , gsu  )
      CALL iom_rstput( fd, fd, inum, 'gru'  , gru  )
      CALL iom_rstput( fd, fd, inum, 'gtv'  , gtv  )
      CALL iom_rstput( fd, fd, inum, 'gsv'  , gsv  )
      CALL iom_rstput( fd, fd, inum, 'grv'  , grv  )
      CALL iom_rstput( fd, fd, inum, 'rn2'  , rn2  )

      CALL iom_close( inum )

   END SUBROUTINE trj_wri_spl

   SUBROUTINE trj_rd_spl(filename)
      !!-----------------------------------------------------------------------
      !!
      !!                  ***  ROUTINE asm_trj__wop_rd ***
      !!
      !! ** Purpose : Read SimPLe data from file the model state trajectory
      !!
      !! ** Method  : 
      !!
      !! ** Action  :
      !!
      !! History : 
      !!        ! 09-07 (F. Vigilant) 
      !!-----------------------------------------------------------------------
      !! *Module udes
      USE iom                 ! I/O module
      !! * Arguments
      !! * Local declarations
      INTEGER :: &
         & inum, &                  ! File unit number
         & fd                       ! field number
      CHARACTER (LEN=50) :: &
         & filename
       
      fd=1
      WRITE(filename, FMT='(A,A)' ) TRIM( filename ), '.nc'
      filename = TRIM( filename )
      CALL iom_open( filename, inum)

      ! Output trajectory fields
      CALL iom_get( inum, jpdom_autoglo, 'un'   , un,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'vn'   , vn,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'tn'   , tn,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'sn'   , sn,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'sshn' , sshn, fd )
      CALL iom_get( inum, jpdom_autoglo, 'wn'   , wn,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'tb'   , tb,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'sb'   , sb,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'ua'   , ua,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'va'   , va,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'ta'   , ta,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'sa'   , sa,   fd )
      CALL iom_get( inum, jpdom_autoglo, 'sshb' , sshb, fd )
      CALL iom_get( inum, jpdom_autoglo, 'rhd'  , rhd,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'rhop' , rhop, fd )
      CALL iom_get( inum, jpdom_autoglo, 'gtu'  , gtu,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'gsu'  , gsu,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'gru'  , gru,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'gtv'  , gtv,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'gsv'  , gsv,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'grv'  , grv,  fd )
      CALL iom_get( inum, jpdom_autoglo, 'rn2'  , rn2,  fd )

      CALL iom_close( inum )

   END SUBROUTINE trj_rd_spl

#endif
END MODULE trj_tam
